package editor;

import gui.Canvas;

import java.awt.Component;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.UIManager;

import org.w3c.dom.Element;

import utils.Global;

/**
 * Represents an editor's button/drop-down menu defined by a name and visibility. 
 * @author hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class Button {
	private boolean bVisible;	    //button's visibility
	private String sTooltip; 	    //button's tooltip text
	private String sXMLname;		//button's short designation as it appears in XML config file
	private Component component;	//Swing object equivalent to button
	               
	/**
	 * Customised constructor. Sets up the button object retrieving its name and visibility from
	 * an XML configuration file. 
	 * @param button XML Element node defining the button in question.
	 */
	public Button (Element button) {
		String sVisible;	//element's content (represents visibility)
		
		this.component = null;
		sXMLname = button.getNodeName();
		sTooltip = Global.LANG.getString (sXMLname);
		sVisible = button.getTextContent();
		if (sVisible.equals ("si")) {
		   bVisible = true;
		} else if (sVisible.equals ("no")) {
			      bVisible = false;
		       } else {
			          System.out.println ("Valor 'visible' incorrecto.");
			          System.exit(1);
		       }
	}
	
	/**
	 * Gets button's name.
	 * @return Name of the button.
	 */
	public String getName () {
		return sXMLname;
	}
	
	/**
	 * Retrieves the Swing component representing the button
	 * @return Swing component
	 */
	public Component getComponent () {
		return component;
	}
	
	/**
	 * Changes the Swing component representing the button
	 * @param newComponent New Swing component for the button
	 */
	public void setComponent (Component newComponent) {
		this.component = newComponent;
	}
	
	/**
	 * Tells whether the button is visible.
	 * @return True if visible. False if not.
	 */
	public Boolean isVisible () {
		return bVisible;
	}
	
	/**
	 * Turns the button represented by this class into an appropriate Swing component. 
	 * @param iIconWidth Button's icon width
	 * @param iIconHeight Button's icon height
	 * @param sIconPath Path to icon image file. 
	 * @param sExtension Icon file extension
	 * @param listener Class managing events from button  
	 */
	public void buttonise (int iIconWidth, int iIconHeight, String sIconPath, 
			               String sExtension, ActionListener listener) {
		this.component = Canvas.toSwing (iIconWidth, iIconHeight, sIconPath, sXMLname, 
				                         sExtension, sTooltip, listener);
	}
	
	/**
	 * Turns the button represented by this class into an appropriate Swing component
	 * @param dropDownItems Items to be displayed in menu
	 * @param listener Class managing events from menu
	 */
	public void buttonise (String[] dropDownItems, ActionListener listener) {
		this.component = Canvas.toSwing (dropDownItems, listener, sXMLname, Global.LANG.getString ("fuenteEtiqueta"));
	}
	
	/**
	 * Updates the selection status of a button when pressed and represents it using 
	 * a different background colour. The new status is the opposite of the former one.
	 */
	public void setSelected () {
		((JButton) component).setSelected (!((JButton) component).isSelected());
		if (((JButton) component).isSelected())
		   ((JButton) component).setBackground (Global.DEFAULT_BORDER);	
		else ((JButton) component).setBackground (UIManager.getColor ("Button.background"));
	}
	
	/**
	 * Updates the selection status of a button when pressed and represents it using 
	 * a different background colour. The new status is specified by the user.
	 * @param bStatus New selection status for the button.
	 */
	public void setSelected (boolean bStatus) {
		((JButton) component).setSelected (bStatus);
		if (bStatus)
		   ((JButton) component).setBackground (Global.DEFAULT_BORDER); 	
		else ((JButton) component).setBackground (UIManager.getColor ("Button.background"));
	}
	
	/**
	 * Enables or disables the component associated with the button.
	 * @param estado True for enabling. False for disabling.
	 */
	public void enable (boolean estado) {
		component.setEnabled (estado);
	}
	
	/**
	 * Returns the selection status of the JButton
	 * @return True if selected. False if not.
	 */
	public boolean isSelected () {
		return ((JButton) component).isSelected();
	}
}
